/*****************************
* Source : piCommon.ch
* System : general
* Author : Phil Ide           
* Created: 13/02/2002
*                             
* Purpose: general purpose command extensions
* ----------------------------
* History:                    
* ----------------------------
*    13/02/2002 08:17 PPI - Created
*    16/06/2003 14:53 PPI - Updated, tidied, added comments
*****************************/

#ifndef __DLLFUNCTION_EXTENSION__
   #define __DLLFUNCTION_EXTENSION__

   // The enhanced DLLFUNCTION command allows you to give
   // an external function any name you like.
   // e.g. you can call a function named "Fred" as "Harry":
   // DLLFUNCTION ShellExec(hWnd,cOp,cFile,cParms,cDir,nShowCmd) ;
   //             ALIAS ShellExecuteA USING STDCALL FROM Shell32.dll
   //
   // Then in your code:
   //    ShellExec(hWnd,"print","mydoc.doc",NIL,CurDir(),SW_HIDE)
   //    /* calls Shell32.dll:ShellExecuteA() */
   //
   // Also, you can attach an expression to the call, which is evaluated
   // against the return value.  This will force a logical value to
   // be returned.  As an example:
   //
   // DLLFUNCTION Harry(nParm) ALIAS Fred USING STDCALL
   //      FROM MyDll.dll RESULT == 4
   //
   // The expression is evaluated replacing RESULT with the return value
   // of the call.  Note that you cannot modify the RESULT value in the
   // expression (that is, you can supply the operand and the rval only).
   // for example, this is illegal:
   //
   //     .... LTrim(RESULT) == "Fred"
   //
   // To perform this test you would have to do this:
   //     .... RESULT == 
   //
   // An extension of this is the AS BOOL keyphrase:
   // DLLFUNCTION Harry(nParm) ALIAS Fred USING STDCALL FROM MyDll.dll AS BOOL
   //
   // This forces a return value of TRUE if the result is non-zero.
   //
   // Note also that this extended version of the command does not unload
   // the DLL, and creates a calling template on first use.  This optimises
   // subsequent calls.
   //
   // If you need to force an unload of the dll, just call DllUnload() as
   // normal.

   // DLLFUNCTION command enhanced
   #command  DLLFUNCTION <Func>([<x,...>]) ALIAS <cAlias>;
                   USING <sys:CDECL,OSAPI,STDCALL,SYSTEM> ;
                    FROM <(Dll)> AS BOOL ;
          => ;
             DLLFUNCTION <Func>([<x>]) ALIAS <cAlias> ;
                   USING <sys> ;
                    FROM <(Dll)> RESULT <> 0


   #command  DLLFUNCTION <Func>([<x,...>]) ALIAS <cAlias>;
                   USING <sys:CDECL,OSAPI,STDCALL,SYSTEM> ;
                    FROM <(Dll)> [RESULT <*c2bExpr*>] ;
          => ;
                FUNCTION <Func>([<x>]);;
                   static cTpl;;
                   static bExpr := {|x| x};;
                   LOCAL nDll:=DllLoad(<(Dll)>);;
                   local xRet;;
                   if cTpl == NIL;;
                      cTpl := DllprepareCall(<(Dll)>,__Sys(<sys>),<(cAlias)>);;
                     [bExpr := {|x| x <c2bExpr>}];;
                   endif;;
                   xRet := DLLExecuteCall( cTpl [,<x>] );;
                   RETURN EVal(bExpr,xRet)



   #command  DLLFUNCTION <Func>([<x,...>]) ALIAS <cAlias>;
                   USING <sys:CDECL,OSAPI,STDCALL,SYSTEM> ;
                    FROM <(Dll)> ;
          => ;
                FUNCTION <Func>([<x>]);;
                   STATIC cTpl;;
                   LOCAL nDll:=DllLoad(<(Dll)>);;
                   LOCAL xRet;;
                   if cTpl == NIL;;
                      cTpl := DllprepareCall(<(Dll)>,__Sys(<sys>),<(cAlias)>);;
                   endif;;
                   xRet := DLLExecuteCall( cTpl [,<x>] );;
                   RETURN xRet

   // the 'standard' Xbase++ version (optimised)
   #command  DLLFUNCTION <Func>([<x,...>]);
                   USING <sys:CDECL,OSAPI,STDCALL,SYSTEM> ;
                    FROM <(Dll)> ;
          => ;
                FUNCTION <Func>([<x>]);;
                   STATIC cTpl;;
                   LOCAL nDll:=DllLoad(<(Dll)>);;
                   LOCAL xRet;;
                   if cTpl == NIL;;
                      cTpl := DllprepareCall(<(Dll)>,__Sys(<sys>),<(Func)>);;
                   endif;;
                   xRet := DLLExecuteCall( cTpl [,<x>] );;
                   RETURN xRet

#endif

#ifndef CRLF
   #define CRLF Chr(13)+Chr(10)
#endif

#ifndef __PI_COMMON_
   #define __PI_COMMON__

   // any type
   #xtranslate (<var> AMONG <n1> [,<n2>])  => (<var>${<n1>[,<n2>]})

   // character
   #xtranslate (<var> SUB <n1> [,<n2>])    => (AScan({<n1>[,<n2>]},{|e| <var>$e }) > 0)
   #xtranslate (<var> IAMONG <n1> [,<n2>])  => (lower(<var>)${lower(<n1>)[,lower(<n2>)]})

   // numeric
   #xtranslate (<var> BETWEEN <n1> ,<n2>)  => ((<var> >= <n1>) .and. (<var> <= <n2>))
   #xtranslate (<var> XBETWEEN <n1> ,<n2>) => ((<var> \> <n1>) .and. (<var> \< <n2>))
   #xtranslate (<var> OUTSIDE <n1> ,<n2>)  => ((<var> \< min(<n1>,<n2>)) .or. (<var> \> max(<n1>,<n2>)))

   // superior type checking
   //
   // these checks can be used in pre-processor statements even when the value
   // to be tested does not exist.
   #xtranslate IsArray(<v>)          =>  ( eval({|x| valtype(x) == 'A' }, <v>) )
   #xtranslate IsBlock(<v>)          =>  ( eval({|x| valtype(x) == 'B' }, <v>) )
   #xtranslate IsChar(<v>)           =>  ( eval({|x| valtype(x) == 'C' }, <v>) )
   #xtranslate IsCharacter(<v>)      =>  ( eval({|x| valtype(x) == 'C' }, <v>) )
   #xtranslate IsDate(<v>)           =>  ( eval({|x| valtype(x) == 'D' }, <v>) )
   #xtranslate IsLogical(<v>)        =>  ( eval({|x| valtype(x) == 'L' }, <v>) )
   #xtranslate IsLog(<v>)            =>  ( eval({|x| valtype(x) == 'L' }, <v>) )
   #xtranslate IsNil(<v>)            =>  ( eval({|x| x == NIL }, <v>) )
   #xtranslate IsNumber(<v>)         =>  ( eval({|x| valtype(x) == 'N' }, <v>) )
   #xtranslate IsNumeric(<v>)        =>  ( eval({|x| valtype(x) == 'N' }, <v>) )
   #xtranslate IsInt(<v>)            =>  ( eval({|x| valtype(x) == 'N' .and. x == int(x) }, <v>) )
   #xtranslate IsInteger(<v>)        =>  ( eval({|x| valtype(x) == 'N' .and. x == int(x) }, <v>) )
   #xtranslate IsMemo(<v>)           =>  ( eval({|x| valtype(x) == 'M' }, <v>) )
   #xtranslate IsObject(<v>)         =>  ( eval({|x| valtype(x) == 'O' }, <v>) )
   #xtranslate IsObj(<v>)            =>  ( eval({|x| valtype(x) == 'O' }, <v>) )
   #xtranslate IsClass(<v>,<class>)  =>  ( eval({|x,y| valtype(x) == 'O' .and. upper(x:className) == upper(y)}, <v>, <class>) )

#endif

